package gonganxitong

import (
	"path"

	"shylinux.com/x/ice"
	icebergs "shylinux.com/x/icebergs"
	"shylinux.com/x/icebergs/base/nfs"
	"shylinux.com/x/icebergs/base/web"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/gonganxitong/model"
	"shylinux.com/x/mysql-story/src/db"
)

type Tabler interface {
	Inputs(m *ice.Message, arg ...string)
	RewriteAppend(m *ice.Message, arg ...string) *ice.Message
	TransValue(m *ice.Message, key string, arg ...string) string
	TransRole(m *ice.Message, arg ...string) string
}
type Container interface {
	FindOrCreateByName(m *ice.Message, arg ...string)
}

type Table struct {
	db.Table
	UserPlace Tabler
	Place     Tabler
	Street    Container
	inputs    string `name:"inputs" role:"void"`
	list      string `name:"list place_uid uid auto" role:"void"`
}

func (s Table) Init(m *ice.Message, arg ...string) {
	kit.If(m.Config(web.PORTAL) == ice.TRUE, func() { Portal{}.Show(m) })
}
func (s Table) Inputs(m *ice.Message, arg ...string) {
	switch arg[0] {
	case model.PLACE_TYPE:
		for k, v := range PlaceTypeList {
			m.Push(arg[0], k).Push(model.NAME, v)
		}
		m.SortInt(arg[0]).DisplayInputKeyNameIconTitle()
	case model.USER_PLACE_ROLE:
		for k, v := range UserPlaceRoleList {
			if k != UserPlaceCreator && k != UserPlaceVisitor {
				m.Push(arg[0], k).Push(model.NAME, v)
			}
		}
		m.SortInt(arg[0]).DisplayInputKeyNameIconTitle()
	default:
		s.Table.Inputs(m, arg...)
	}
}
func (s Table) List(m *ice.Message, arg ...string) {
}
func (s Table) TablesWithRole(m *ice.Message, arg []string, target, userPlace, place Tabler, fields ...ice.Any) *ice.Message {
	s.Tables(m, kit.JoinWord("LEFT JOIN", s.TableName(kit.TypeName(userPlace)),
		"ON", s.Key(target, model.USER_UID), "=", s.Key(userPlace, model.USER_UID),
		"AND", s.Key(target, s.Keys(place, model.UID)), "=", s.Key(userPlace, s.Keys(place, model.UID)),
	)).FieldsWithCreatedAT(m, target, append([]ice.Any{s.Key(target, model.USER_UID), s.AS(s.Key(userPlace, model.ROLE), s.Keys(userPlace, model.ROLE))}, fields...)...)
	if len(arg) == 1 {
		s.Table.Select(m, s.Key(target, s.Keys(place, model.UID)), arg[0])
	} else if len(arg) == 2 {
		s.Table.Select(m, s.Key(target, s.Keys(place, model.UID)), arg[0], s.Key(target, model.UID), arg[1])
	} else {
		return m
	}
	userPlace.RewriteAppend(m)
	return s.SelectJoinUser(m)
}
func (s Table) SelectJoinUser(m *ice.Message) *ice.Message {
	return s.SelectJoin(m, user{}, model.NAME, model.AVATAR)
}
func (s Table) SelectJoinCity(m *ice.Message) *ice.Message {
	return s.SelectJoin(m, city{}, model.NAME)
}
func (s Table) RewriteAppend(m *ice.Message, arg ...string) *ice.Message {
	m.RewriteAppend(func(value, key string, index int) string {
		kit.If(key == model.PLACE_TYPE, func() { value = PlaceType(kit.Int(value)).String() })
		kit.If(key == model.USER_PLACE_ROLE, func() { value = UserPlaceRole(kit.Int(value)).String() })
		kit.If(key == model.ORDER_STATUS, func() { value = OrderStatus(kit.Int(value)).String() })
		kit.If(key == model.APPLY_STATUS, func() { value = ApplyStatus(kit.Int(value)).String() })
		return value
	})
	return m
}
func (s Table) RenameAppend(m *ice.Message, arg ...string) Table {
	m.RenameAppend(arg...)
	return s
}
func (s Table) TransValue(m *ice.Message, key string, arg ...string) string {
	if value := kit.Select(m.Option(key), arg, 0); m.IsEnglish() {
		return value
	} else {
		return kit.Select(value, kit.Value(m.Target().Commands[m.CommandKey()].Meta, "_trans.value."+key+"."+value))
	}
}
func (s Table) TransRole(m *ice.Message, arg ...string) string {
	value := kit.Select(m.Option(s.Keys(s.UserPlace, model.ROLE)), arg, 0)
	role := s.Place.TransValue(m, s.Keys(s.UserPlace, model.ROLE), arg...)
	return kit.Format(`<span class="role %s">%s</span>`, value, role)
}
func (s Table) SendTemplate(m *ice.Message, from, user_uid, title string) {
	if !m.IsErr() {
		m.Cmd(user{}, user{}.SendTemplate, from, user_uid, Portal{}.Link(m, m.Option(s.Keys(s.Place, model.UID)), m.PrefixKey(), m.Option(model.UID)),
			title, m.Option(s.Keys(s.Place, model.NAME))+" "+s.Place.TransValue(m, s.Keys(s.UserPlace, model.ROLE)), kit.Cut(m.Option(model.UID), 6))
	}
}
func (s Table) Display(m *ice.Message) *ice.Message {
	m.Option("_place_uid", s.ToLower(kit.TypeName(s.Place))+"_uid")
	m.Option("_place_name", s.ToLower(kit.TypeName(s.Place))+"_name")
	m.Option("_place_type", s.ToLower(kit.TypeName(s.Place))+"_type")
	m.Option("_user_place_role", s.ToLower(kit.TypeName(s.UserPlace))+"_role")
	m.Option("_street_name", s.ToLower(kit.TypeName(s.Street))+"_name")
	file := ""
	kit.If(file == "", func() { file = m.CommandKey() + ".js" })
	if !kit.HasPrefix(file, nfs.PS, web.HTTP) {
		file = m.Resource(path.Join(path.Dir(kit.FileLine(1, 100)), file))
	}
	m.Display(file)
	return m
}

type Tables struct {
	Table
	Portal Portal
	portal string `data:"true"`
}

func (s Tables) Init(m *ice.Message, arg ...string) {
	s.Portal.Show(m)
}
func (s Tables) Link(m *ice.Message, arg ...string) string {
	return s.Portal.Link(m, arg...)
}
func (s Tables) BeforeMigrate(m *ice.Message, arg ...string) {
}

func newTables() Tables {
	return Tables{Table: newTable()}
}
func newTable() Table {
	return Table{UserPlace: userPlace{}, Place: place{}, Street: street{}}
}
func NewTable(userPlace, place Tabler, street Container) Table {
	return Table{UserPlace: userPlace, Place: place, Street: street}
}
func NewPortal(userPlace, place Tabler, street Container) Portal {
	return Portal{Table: Table{UserPlace: userPlace, Place: place, Street: street}}
}
func (s Portal) getTable() Table {
	return s.Table
}
func PortalCmd(portal ice.Any) {
	h := kit.FileLine(-1, 100)
	p := kit.Keys("web.team", kit.ModPath(-1))
	cmd := func(name string, data ice.Any) {
		_, cmd := ice.Cmd(kit.Keys(p, name), data)
		cmd.RawHand = path.Join(path.Dir(h), name+".go")
		h := cmd.Actions["beforeMigrate"].Hand
		cmd.Actions["beforeMigrate"].Hand = func(m *icebergs.Message, arg ...string) {
			ice.LoadTrans(m.Cmd("web.team.gonganxitong.apply"), cmd)
			ice.LoadTrans(m, cmd)
			h(m, arg...)
		}
	}
	table := portal.(interface{ getTable() Table }).getTable()
	cmd("portal", portal)
	cmd("qrcode", qrcode{Tables: Tables{Table: table}})
	cmd("apply", apply{Table: table})
	cmd("order", order{Table: table})
	cmd("event", event{Table: table})
	cmd("service", service{})
}
